'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _carousel = require('../carousel');

var _carousel2 = _interopRequireDefault(_carousel);

var _carouselItem = require('../carousel-item');

var _carouselItem2 = _interopRequireDefault(_carouselItem);

var _calendar = require('../mixins/calendar');

var _calendar2 = _interopRequireDefault(_calendar);

var _util = require('../calendar/util');

var _dateUtil = require('../utils/dateUtil');

var _event = require('../_utils/dom/event');

var _raf = require('../_utils/dom/raf');

require('./index.less');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var Calendar = {
    name: 'SnCalendar',
    mixins: [_calendar2['default']],
    data: function data() {
        return {
            height: 0,
            weekList: ['日', '一', '二', '三', '四', '五', '六'],
            year: 0,
            month: 0,
            currentValue: '',
            currentDate: '',
            swiperList: [],
            swiperItemIndex: 0,
            minDate: false,
            maxDate: false
        };
    },

    computed: {
        styleHeight: function styleHeight() {
            return this.height + 'px';
        }
    },
    watch: {
        value: function value(val) {
            this.currentValue = val;
        }
    },
    created: function created() {
        this.currentValue = this.value;
        var date = void 0,
            year = void 0,
            month = void 0;
        date = this.currentValue && new Date(this.currentValue) || new Date();
        year = date.getFullYear();
        month = date.getMonth();

        this.currentValue = (0, _dateUtil.dateFormat)(date, 'YYYY/MM/DD');
        this.render(year, month);
        this.showTimeSelect = true;

        /* 动态计算swiper高度 */
        this.getHeight();
    },
    mounted: function mounted() {},

    methods: {
        /* 获取高度 */
        getHeight: function getHeight() {
            var windowWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
            this.height = windowWidth < 480 ? windowWidth * 0.94 : 360;
        },

        /* 上一页 */
        prev: function prev() {
            if (!this.minDate) {
                var swiperItemIndex = this.swiperItemIndex;
                var goIndex = swiperItemIndex - 1 >= 0 ? swiperItemIndex - 1 : 2;
                this.$refs.carousel.swipeTo(goIndex, { immediate: true });
            }
        },

        /* 下一页 */
        next: function next() {
            if (!this.maxDate) {
                var swiperItemIndex = this.swiperItemIndex;
                var goIndex = swiperItemIndex + 1 <= 2 ? swiperItemIndex + 1 : 0;
                this.$refs.carousel.swipeTo(goIndex, { immediate: true });
            }
        },

        /* 跳转页 */
        go: function go(year, month) {
            this.render(year, month, true);
        },

        /* 选择日期 */
        select: function select(event, k1, k2, data) {
            (0, _event.preventDefault)(event, true);
            var showLastMonth = this.showLastMonth,
                showNextMonth = this.showNextMonth;


            if (data.isDisabled) {
                return;
            }
            if (!showLastMonth && data.isLastMonth) {
                return;
            }
            if (!showNextMonth && data.isNextMonth) {
                return;
            }
            var _currentValue = data.formatedDate;
            if ((!data.isLastMonth || showLastMonth) && (!data.isNextMonth || showNextMonth)) {
                var index = this.swiperItemIndex;
                var month = this.swiperList[index];
                var days = month.days;
                for (var row = 0; row < days.length; row++) {
                    for (var day = 0; day < days[row].length; day++) {
                        if (row === k1 && day === k2) {
                            days[k1][k2].isCurrent = true;
                        } else {
                            days[row][day].isCurrent = false;
                        }
                    }
                }
                if (showLastMonth && data.isLastMonth) this.prev();
                if (showNextMonth && data.isNextMonth) this.next();
            }
            this.currentValue = _currentValue;
            this.currentDate = _currentValue;

            if (this.renderOnValueChange) {
                this.render(null, null);
            }

            this.$emit('select', this.currentValue, this.currentDate);
        },

        /* 设置选中的天 */
        setSelectedDay: function setSelectedDay() {
            var currentDate = new Date(this.currentValue);
            var day = currentDate.getDate();
            /* 当前月最大日期 */
            var selectedMonthMaxDate = new Date(this.year, this.month + 1, 0).getDate();
            var selectedDay = (0, _dateUtil.dateFormat)(new Date(this.year, this.month, day < selectedMonthMaxDate ? day : selectedMonthMaxDate), 'YYYY/MM/DD');
            this.currentValue = selectedDay;
            this.currentDate = selectedDay;
            for (var t = 0; t < this.swiperList.length; t++) {
                var month = this.swiperList[t];
                var days = month.days;
                for (var row = 0; row < days.length; row++) {
                    for (var _day = 0; _day < days[row].length; _day++) {
                        if (days[row][_day].formatedDate === selectedDay) {
                            this.swiperList[t].days[row][_day].isCurrent = true;
                        } else {
                            this.swiperList[t].days[row][_day].isCurrent = false;
                        }
                    }
                }
            }
        },

        /* 日历onChange回调 */
        onCalendarChange: function onCalendarChange(currentIndex) {
            var _this = this;

            /* 滑动结束后执行选中滑动前选中的天 */
            if (currentIndex !== this.swiperItemIndex) {
                this.$nextTick(function () {
                    (0, _raf.doubleRaf)(function () {
                        var disabledDate = _this.disabledDate;

                        var _year = _this.year,
                            _month = _this.month,
                            _currentValue = _this.currentValue;

                        _this.year = _this.swiperList[currentIndex].year;
                        _this.month = _this.swiperList[currentIndex].month;

                        var prevTime = (0, _util.getPrevTime)(_year, _month);
                        var prev2Time = (0, _util.getPrevTime)(prevTime.year, prevTime.month);
                        var nextTime = (0, _util.getNextTime)(_year, _month);
                        var next2Time = (0, _util.getNextTime)(nextTime.year, nextTime.month);

                        var prev2Month = (0, _util.getDays)((0, _extends3['default'])({}, prev2Time, {
                            value: _currentValue,
                            disabledDate: disabledDate
                        }));
                        var next2Month = (0, _util.getDays)((0, _extends3['default'])({}, next2Time, {
                            value: _currentValue,
                            disabledDate: disabledDate
                        }));

                        /*  数据防抖原理：初始化时，当前处于第一页，考虑有下滑，上滑动作，那么下滑对应的页面是第二页，
                            即设置下一月的数据，上滑对应的页面是第三页，即设置上一月的数据；在滑动完时，立马修改当前
                            所对应的下一页，上一页的数据，循环利用当前数组，那么无论上滑还是下滑，数据都是提前修改好
                            的，对于用户来说，数据则不会产生抖动。
                            eg:
                            当前页为第一页
                                下滑，进入第二页; 开始顺序 => [cur, next, prev], 结束顺序 => [cur, next, next2]
                                上滑，进入第三页; 开始顺序 => [cur, next, prev], 结束顺序 => [cur, prev2, prev]
                            当前页为第二页
                                下滑，进入第三页; 开始顺序 => [prev, cur, next], 结束顺序 => [next2, cur, next]
                                上滑，进入第一页; 开始顺序 => [prev, cur, next], 结束顺序 => [prev, cur, prev2]
                            当前页为第三页
                                下滑，进入第一页; 开始顺序 => [next, prev, cur], 结束顺序 => [next, next2, cur]
                                上滑，进入第二页; 开始顺序 => [next, prev, cur], 结束顺序 => [prev2, prev, cur]
                        */

                        if (_this.swiperItemIndex === (currentIndex + 2) % 3) {
                            // 下滑
                            _this.$set(_this.swiperList, (_this.swiperItemIndex + 2) % 3, next2Month);
                        } else {
                            // 上滑
                            _this.$set(_this.swiperList, (currentIndex + 2) % 3, prev2Month);
                        }

                        _this.swiperItemIndex = currentIndex;

                        _this.setSelectedDay();
                        _this.$emit('calendar-change', _this.currentValue);
                    });
                });
            }
        },

        /* 渲染入口 */
        render: function render(year, month) {
            var force = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

            var value = this.currentValue;

            this.year = year;
            this.month = month;

            /* 注入滑动数组 */
            this.renderCarousel(year, month, value);
        },

        /* 渲染日历数据 */
        renderCarousel: function renderCarousel(year, month, value) {
            var _this2 = this;

            var disabledDate = this.disabledDate;

            /* 生成总共需要的四个月数据 */

            var prevTime = (0, _util.getPrevTime)(year, month);
            var nextTime = (0, _util.getNextTime)(year, month);
            var prevMonth = (0, _util.getDays)((0, _extends3['default'])({}, prevTime, { value: value, disabledDate: disabledDate }));
            var curMonth = (0, _util.getDays)({ year: year, month: month, value: value, disabledDate: disabledDate });
            var nextMonth = (0, _util.getDays)((0, _extends3['default'])({}, nextTime, { value: value, disabledDate: disabledDate }));

            this.$refs.carousel && this.$refs.carousel.swipeTo(0, { immediate: true });
            this.swiperItemIndex = 0;
            (0, _raf.doubleRaf)(function () {
                _this2.swiperList = [];
                _this2.swiperList = [curMonth, nextMonth, prevMonth];

                _this2.setSelectedDay();
            });
        },

        /* 渲染日历月份 */
        renderCalendarWeekday: function renderCalendarWeekday() {
            var h = this.$createElement;
            var weekList = this.weekList;


            return h(
                'div',
                { 'class': 'sn-calendar-weekday' },
                [weekList.map(function (week) {
                    return h(
                        'div',
                        { 'class': 'sn-calendar-weekday-item' },
                        [h('div', [week])]
                    );
                })]
            );
        },

        /* 渲染日历月份 */
        renderCalendarMonth: function renderCalendarMonth(month) {
            var _this3 = this;

            var h = this.$createElement;

            return h(
                'div',
                { 'class': 'sn-calendar-month' },
                [month.days.map(function (row, k1) {
                    return h(
                        'div',
                        { 'class': 'month-row' },
                        [row.map(function (item, k2) {
                            return _this3.renderCalendarDay(item, k1, k2);
                        })]
                    );
                })]
            );
        },

        /* 渲染日历日期 */
        renderCalendarDay: function renderCalendarDay(item, k1, k2) {
            var _this4 = this;

            var h = this.$createElement;
            var showLastMonth = this.showLastMonth,
                showNextMonth = this.showNextMonth,
                workDate = this.workDate,
                infoDate = this.infoDate;


            var workingDay = void 0,
                infoDay = void 0;
            if (Array.isArray(workDate) && workDate.length) {
                for (var i in workDate) {
                    if (workDate[i].date === item.formatedDate) {
                        workingDay = workDate[i];
                    }
                }
            }
            if (Array.isArray(infoDate) && infoDate.length) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = infoDate[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var infoItem = _step.value;

                        if (infoItem && typeof infoItem === 'string') {
                            if (infoItem === item.formatedDate) {
                                infoDay = {
                                    date: infoItem
                                };
                            }
                        } else {
                            if (infoItem.date === item.formatedDate) {
                                infoDay = infoItem;
                            }
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator['return']) {
                            _iterator['return']();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            }

            // 日期头部（适用于工作日历）
            var renderDayHeader = h(
                'div',
                { 'class': 'header' },
                [workingDay ? workingDay.isWorking ? h(
                    'span',
                    { 'class': 'working' },
                    ['\u73ED']
                ) : h(
                    'span',
                    { 'class': 'rest' },
                    ['\u4F11']
                ) : null]
            );

            // 日期尾部（适用于节假日）
            var renderDayFooter = h(
                'div',
                { 'class': 'footer' },
                [infoDay ? h('span', {
                    'class': (0, _defineProperty3['default'])({
                        'info-dot': true
                    }, infoDay['class'], infoDay['class']),
                    style: {
                        borderColor: infoDay.color ? infoDay.color : ''
                    }
                }) : null]
            );

            return h(
                'div',
                {
                    'class': {
                        day: true,
                        'day-in-last-month': item.isLastMonth,
                        'day-in-next-month': item.isNextMonth
                    }
                },
                [item.isCurMonth || item.isLastMonth && showLastMonth || item.isNextMonth && showNextMonth ? h(
                    'div',
                    {
                        'class': 'day-pack',
                        on: {
                            'click': function click(event) {
                                return _this4.select(event, k1, k2, item);
                            }
                        }
                    },
                    [h(
                        'div',
                        {
                            'class': {
                                today: item.isToday,
                                active: item.isCurrent,
                                disabled: item.isDisabled
                            }
                        },
                        [h(
                            'div',
                            { 'class': 'day-pack-cell' },
                            [renderDayHeader, h(
                                'div',
                                {
                                    'class': {
                                        middle: true,
                                        isHoliday: item.isHoliday,
                                        hidden: item.dayHide
                                    }
                                },
                                [item.isToday ? '今' : item.day]
                            ), renderDayFooter]
                        )]
                    )]
                ) : null]
            );
        }
    },

    render: function render() {
        var _this5 = this;

        var h = arguments[0];
        var vertical = this.vertical,
            touchable = this.touchable,
            mouseable = this.mouseable,
            styleHeight = this.styleHeight,
            swiperList = this.swiperList;


        return h(
            'div',
            { 'class': 'sn-calendar' },
            [this.renderCalendarWeekday(), h(
                'div',
                { 'class': 'sn-calendar-panel', style: { height: styleHeight } },
                [h(
                    _carousel2['default'],
                    {
                        ref: 'carousel',
                        attrs: { 'show-indicators': false,
                            vertical: vertical,
                            touchable: touchable,
                            mouseable: mouseable,
                            threshold: 0
                        },
                        style: { height: styleHeight },
                        on: {
                            'change': this.onCalendarChange
                        }
                    },
                    [swiperList.map(function (month) {
                        return h(
                            _carouselItem2['default'],
                            { style: { height: styleHeight } },
                            [h('div', [h(
                                'div',
                                { 'class': 'sn-calendar-date' },
                                [month.year + '-' + month.month_str]
                            ), _this5.renderCalendarMonth(month)])]
                        );
                    })]
                )]
            )]
        );
    }
};

Calendar.install = function (Vue) {
    Vue.component(Calendar.name, Calendar);
};

exports['default'] = Calendar;
module.exports = exports.default;