'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _calendar = require('../calendar');

var _calendar2 = _interopRequireDefault(_calendar);

var _pickerView = require('../picker-view');

var _pickerView2 = _interopRequireDefault(_pickerView);

var _button = require('../button');

var _button2 = _interopRequireDefault(_button);

var _calendar3 = require('../mixins/calendar');

var _calendar4 = _interopRequireDefault(_calendar3);

var _dateUtil = require('../utils/dateUtil');

var _numberUtil = require('../utils/numberUtil');

var _event = require('../_utils/dom/event');

require('./index.less');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var DateTimePickerView = {
    name: 'SnDatetimePickerView',
    mixins: [_calendar4['default']],
    data: function data() {
        return {
            height: 0,
            panelTitle: '',
            times: [],
            noons: [],
            selectedDate: [],
            selectedTime: [],
            weekList: ['日', '一', '二', '三', '四', '五', '六'],
            yearAndMonthList: [],
            showCalender: true,
            showMonthSelect: false,
            showTimeSelect: false,
            showNoonSelect: false,
            year: 0,
            month: 0,
            currentValue: '',
            currentDate: '',
            currentTime: '08:30',
            currentNoon: '上午',
            swiperItemIndex: 0,
            minDate: false,
            maxDate: false
        };
    },

    props: {
        /** 时间控件模式
         * 1) 'date':         选择日期
         * 2) 'datetime':     选择时间
         * 3) 'year-month':   选择年月
         * 4）'year':         选择年份
         * 5）'month':        选择月份
         * 6) 'time':         选择时分
         */
        mode: {
            type: String,
            'default': 'date',
            validator: function validator(value) {
                return ['date', 'datetime', 'year-month', 'year', 'month', 'time'].indexOf(value) > -1;
            }
        },
        minuteStep: [String, Number],
        use12Hours: Boolean,
        noon: {
            type: String,
            'default': '上午',
            validator: function validator(value) {
                return ['上午', '下午'].indexOf(value) > -1;
            }
        },
        panelTitleFormat: {
            type: String,
            'default': 'YYYY年MM月DD日'
        },
        rangeHours: Array,
        rangeMinutes: Array
    },
    computed: {
        styleHeight: function styleHeight() {
            return this.height + 'px';
        }
    },
    watch: {
        value: function value(val) {
            this.currentValue = val;
        },
        currentValue: function currentValue(val) {
            var mode = this.mode,
                currentValue = this.currentValue,
                selectedDate = this.selectedDate,
                use12Hours = this.use12Hours,
                noon = this.noon,
                panelTitleFormat = this.panelTitleFormat;

            if (mode === 'date' || mode === 'datetime' || mode === 'year-month') {
                if (mode === 'date') {
                    this.panelTitle = (0, _dateUtil.dateFormat)(currentValue, panelTitleFormat);
                } else if (mode === 'datetime') {
                    this.panelTitle = (0, _dateUtil.dateFormat)(currentValue, panelTitleFormat);
                } else {
                    this.panelTitle = selectedDate[0] + '\u5E74' + selectedDate[1] + '\u6708';
                }
            } else if (mode === 'year') {
                this.panelTitle = selectedDate[0] + '\u5E74';
            } else if (mode === 'month') {
                this.panelTitle = selectedDate[0] + '\u6708';
            }
        },
        selectedDate: function selectedDate(val) {
            var mode = this.mode;

            if (mode === 'date' || mode === 'datetime' || mode === 'year-month') {
                var now = new Date();

                if (parseInt(val[0]) === 1980 && parseInt(val[1]) === 1) {
                    this.minDate = true;
                }
                if (parseInt(val[0]) === now.getFullYear() + 20 && parseInt(val[1]) === 12) {
                    this.maxDate = true;
                }

                if (mode === 'year-month') {
                    this.panelTitle = val[0] + '\u5E74' + val[1] + '\u6708';
                }
            }
        },
        selectedTime: function selectedTime(val) {
            var mode = this.mode;

            if (mode === 'time') {
                this.panelTitle = val[0] + '\u65F6' + val[1] + '\u5206';
            }
        }
    },
    created: function created() {
        this.currentValue = this.value;
        var mode = this.mode;
        var date = void 0,
            year = void 0,
            month = void 0;
        if (Array.isArray(this.currentValue)) {
            if (mode === 'year-month') {
                if (this.currentValue.length === 0) {
                    year = new Date().getFullYear();
                    month = new Date().getMonth();
                } else if (this.currentValue.length === 1) {
                    year = Number(this.currentValue[0]);
                    month = new Date().getMonth();
                } else if (this.currentValue.length === 2) {
                    year = Number(this.currentValue[0]);
                    month = Number(this.currentValue[1]) - 1;
                }
            } else if (mode === 'year') {
                if (this.currentValue.length === 0) {
                    year = new Date().getMonth();
                } else if (this.currentValue.length === 1) {
                    year = Number(this.currentValue[0]);
                }
            } else if (mode === 'month') {
                if (this.currentValue.length === 0) {
                    month = new Date().getMonth();
                } else if (this.currentValue.length === 1) {
                    month = Number(this.currentValue[0]) - 1;
                }
            }
        } else {
            date = this.currentValue && new Date(this.currentValue) || new Date();
            year = date.getFullYear();
            month = date.getMonth();
        }
        var fullYear = '' + year,
            fullMonth = (0, _numberUtil.zeroComplete)(month + 1);
        if (mode === 'date') {
            this.currentValue = (0, _dateUtil.dateFormat)(date, 'YYYY/MM/DD');
            this.selectedDate = [fullYear, fullMonth];
            this.genYearAndMonth(year, month);

            if (this.use12Hours) {
                this.genNoon(this.noon);
            }
        } else if (mode === 'datetime') {
            try {
                this.currentDate = (0, _dateUtil.dateFormat)(date, 'YYYY/MM/DD');
            } catch (e) {
                throw Error('[sinosun-ui] datetimePickerView 时间格式不正确');
            }
            this.currentValue = (0, _dateUtil.dateFormat)(date, 'YYYY/MM/DD');
            this.selectedDate = [fullYear, fullMonth];
            this.genYearAndMonth(year, month);
            this.genTime(date.getHours(), date.getMinutes());
        } else if (mode === 'year-month') {
            this.showCalender = false;
            this.showMonthSelect = true;
            this.selectedDate = [fullYear, fullMonth];
            this.genYearAndMonth(year, month);
        } else if (mode === 'year') {
            this.showCalender = false;
            this.showMonthSelect = true;
            this.selectedDate = [fullYear];
            this.genYearAndMonth(year);
        } else if (mode === 'month') {
            this.showCalender = false;
            this.showMonthSelect = true;
            this.selectedDate = [fullMonth];
            this.genYearAndMonth(null, month);
        } else if (mode === 'time') {
            this.showCalender = false;
            this.showMonthSelect = false;
            this.showTimeSelect = true;
            if (this.currentValue) {
                if (Array.isArray(this.currentValue)) {
                    this.genTime(this.currentValue[0], this.currentValue[1]);
                } else {
                    console.error('[sinosun-ui] DatetimePickerView 模式为time时，value值必须为数组。');
                }
            } else {
                this.genTime(date.getHours(), date.getMinutes());
            }
        }
    },
    mounted: function mounted() {
        this.getCalendarHeight();

        if (['year-month', 'year', 'month'].indexOf(this.mode) > -1) {
            this.$refs.yearMonthPicker.setValues(this.selectedDate);
        }
        if (['time'].indexOf(this.mode) > -1) {
            this.$refs.timePicker.setValues(this.selectedTime);
        }
    },

    methods: {
        /* 获取日历高度 */
        getCalendarHeight: function getCalendarHeight() {
            var calendarRef = this.$refs['calendar'];
            if (calendarRef) {
                this.height = calendarRef.$el.offsetHeight;
            } else {
                var windowWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
                this.height = windowWidth < 480 ? windowWidth * 0.94 : 360;
            }
        },


        /* 选择日期 */
        select: function select(value, date) {
            var mode = this.mode,
                use12Hours = this.use12Hours;


            this.currentValue = value;
            this.currentDate = date;

            if (mode === 'date') {
                if (use12Hours) {
                    this.onViewChange('noon');
                    this.$emit('change', value, this.currentNoon);
                } else {
                    this.$emit('input', value);
                    this.$emit('change', value);
                }
            } else if (mode === 'datetime') {
                this.onViewChange('time');
                this.$emit('change', date + ' ' + this.currentTime);
            }
        },


        /* 确认日期 */
        submit: function submit(event) {
            (0, _event.preventDefault)(event, true);
            var mode = this.mode;
            if (mode === 'datetime') {
                this.currentValue = this.currentDate + ' ' + this.currentTime;
            } else if (mode === 'year-month') {
                this.currentValue = this.selectedDate;
            } else if (mode === 'year' || mode === 'month') {
                this.currentValue = this.selectedDate;
            } else if (mode === 'time') {
                this.currentValue = this.selectedTime;
            }
            if (this.use12Hours) {
                this.$emit('input', this.currentValue, this.currentNoon);
            } else {
                this.$emit('input', this.currentValue);
            }
        },


        /* 日历onChange回调 */
        onCalendarChange: function onCalendarChange(value) {
            this.currentValue = value;
            this.$emit('calendar-change', this.currentValue);
        },


        /* 年月onChange回调 */
        onYearAndMonthChange: function onYearAndMonthChange(picker, value, index) {
            this.selectedDate = value;
            if (Array.isArray(value)) {
                var mode = this.mode;

                if (mode === 'date' || mode === 'datetime' || mode === 'year-month') {
                    var year = value[0];
                    var month = value[1];

                    if (mode !== 'year-month') {
                        var day = new Date(this.currentValue).getDate();
                        /* 当前月最大日期 */
                        var selectedMonthMaxDate = new Date(year, month, 0).getDate();
                        var selectedDay = (0, _dateUtil.dateFormat)(new Date(year, month - 1, day < selectedMonthMaxDate ? day : selectedMonthMaxDate), 'YYYY/MM/DD');
                        this.currentValue = selectedDay;
                        this.currentDate = selectedDay;
                        this.$refs['calendar'].render(parseInt(year), parseInt(month - 1), true);
                    }
                } else if (mode === 'year') {
                    var _year2 = value[0];
                    this.panelTitle = _year2 + '\u5E74';
                } else if (mode === 'month') {
                    var _month = value[0];
                    this.panelTitle = _month + '\u6708';
                }

                this.$emit('change', value);
            }
        },


        /* 时间onChange回调 */
        onTimeChange: function onTimeChange(picker, value, index) {
            this.selectedTime = value;
            this.currentTime = value[0] + ':' + value[1];
        },


        /* 上下午onChange回调 */
        onNoonChange: function onNoonChange(picker, value, index) {
            this.currentNoon = value[0];
        },


        /* 切换视图 */
        onViewChange: function onViewChange(type) {
            var _this = this;

            if (this.mode === 'date' || this.mode === 'datetime') {
                var invokePicker = function invokePicker(type) {
                    _this.showCalender = false;

                    if (type === 'date') {
                        _this.showMonthSelect = true;
                        _this.$refs.yearMonthPicker.setValues(_this.selectedDate);
                    } else if (type === 'time') {
                        _this.showTimeSelect = true;
                        _this.$refs.timePicker.setValues(_this.selectedTime);
                    } else if (type === 'noon') {
                        _this.showNoonSelect = true;
                        _this.$refs.noonPicker.setValues([_this.currentNoon]);
                    } else {
                        _this.showCalender = true;
                    }
                };

                if (this.showMonthSelect) {
                    this.showMonthSelect = false;
                    invokePicker(type);
                } else if (this.showTimeSelect) {
                    this.showCalender = true;
                    this.showMonthSelect = false;
                    this.showTimeSelect = false;
                } else if (this.showNoonSelect) {
                    this.showCalender = true;
                    this.showMonthSelect = false;
                    this.showNoonSelect = false;
                } else {
                    invokePicker(type);
                }
            }
        },


        /* 生成年月数据并选中值 */
        genYearAndMonth: function genYearAndMonth(year, month) {
            var mode = this.mode;
            var _date = new Date();
            var _year = _date.getFullYear();
            var yearList = [],
                monthList = [];
            // 加载自1980至当前后20年数据
            for (var i = 1980; i <= _year + 20; i++) {
                yearList.push('' + i);
            }
            // 加载12个月份
            for (var _i = 1; _i <= 12; _i++) {
                var _month2 = (0, _numberUtil.zeroComplete)(_i);
                monthList.push('' + _month2);
            }

            if (mode === 'date' || mode === 'datetime' || mode === 'year-month') {
                this.yearAndMonthList = [{
                    values: yearList,
                    defaultIndex: 0
                }, {
                    values: monthList,
                    defaultIndex: 1
                }];
                this.selectedDate = ['' + year, '' + (0, _numberUtil.zeroComplete)(month + 1)];
            } else if (mode === 'year') {
                this.yearAndMonthList = [{ values: yearList }];
                this.selectedDate = ['' + year];
            } else if (mode === 'month') {
                this.yearAndMonthList = [{ values: monthList }];
                this.selectedDate = ['' + (0, _numberUtil.zeroComplete)(month + 1)];
            }
        },


        /* 生成时分数据并选中值 */
        genTime: function genTime(hour, min) {
            var now = new Date();

            var hours = [];
            var minutes = [];
            for (var i = 0; i < 60; i++) {
                if (i < 24) {
                    hours.push('' + (0, _numberUtil.zeroComplete)(i));
                }

                if (this.minuteStep !== undefined) {
                    i % this.minuteStep === 0 && minutes.push('' + (0, _numberUtil.zeroComplete)(i));
                } else {
                    minutes.push('' + (0, _numberUtil.zeroComplete)(i));
                }
            }

            // 如果存在自定义小时，则以自定义为准
            if (Array.isArray(this.rangeHours) && this.rangeHours.length) {
                hours = [];
                for (var _i2 = 0; _i2 < this.rangeHours.length; _i2++) {
                    hours.push('' + (0, _numberUtil.zeroComplete)(this.rangeHours[_i2]));
                }
            }

            // 如果存在自定义分钟，则以自定义为准
            if (Array.isArray(this.rangeMinutes) && this.rangeMinutes.length) {
                minutes = [];
                for (var _i3 = 0; _i3 < this.rangeMinutes.length; _i3++) {
                    minutes.push('' + (0, _numberUtil.zeroComplete)(this.rangeMinutes[_i3]));
                }
            }

            this.times = [{
                values: hours,
                defaultIndex: 0
            }, {
                values: minutes,
                defaultIndex: 1
            }];

            this.selectedTime = [(0, _numberUtil.zeroComplete)(!isNaN(hour) ? hour : now.getHours()), (0, _numberUtil.zeroComplete)(!isNaN(min) ? min : now.getMinutes())];

            // 如果有自定义时分，默认选中为首项
            if (Array.isArray(this.rangeHours) && this.rangeHours.length) {
                this.selectedTime[0] = (0, _numberUtil.zeroComplete)(this.value && this.rangeHours.includes(hour) ? hour : this.rangeHours[0]);
            }
            if (Array.isArray(this.rangeMinutes) && this.rangeMinutes.length) {
                this.selectedTime[1] = (0, _numberUtil.zeroComplete)(this.value && this.rangeMinutes.includes(min) ? min : this.rangeMinutes[0]);
            }

            this.currentTime = this.selectedTime.join(':');
        },


        /* 生成上下午数据并选中值 */
        genNoon: function genNoon(noon) {
            this.noons = [{
                values: ['上午', '下午']
            }];
            this.currentNoon = noon;
        },


        /* 渲染头部 */
        renderHeader: function renderHeader() {
            var _this2 = this;

            var h = this.$createElement;
            var mode = this.mode,
                use12Hours = this.use12Hours,
                panelTitle = this.panelTitle,
                currentTime = this.currentTime,
                currentNoon = this.currentNoon,
                showCalender = this.showCalender,
                showMonthSelect = this.showMonthSelect,
                showTimeSelect = this.showTimeSelect,
                showNoonSelect = this.showNoonSelect;


            return h(
                'div',
                { 'class': 'sn-datetime-picker-view-header' },
                [h(
                    'div',
                    { 'class': 'sn-datetime-picker-view-tabs' },
                    [h(
                        'div',
                        {
                            'class': {
                                'sn-datetime-picker-view-tabs-item': true,
                                active: (mode === 'date' || mode === 'datetime') && showMonthSelect
                            },
                            on: {
                                'click': function click() {
                                    return _this2.onViewChange(showCalender ? 'date' : null);
                                }
                            }
                        },
                        [panelTitle]
                    ), mode === 'datetime' ? h(
                        'div',
                        {
                            'class': {
                                'sn-datetime-picker-view-tabs-item': true,
                                active: showTimeSelect,
                                'replace-arrow-seat': showMonthSelect || showTimeSelect
                            },
                            on: {
                                'click': function click() {
                                    return _this2.onViewChange('time');
                                }
                            }
                        },
                        [currentTime]
                    ) : null, use12Hours ? h(
                        'div',
                        {
                            'class': {
                                'sn-datetime-picker-view-tabs-item': true,
                                active: showNoonSelect,
                                'replace-arrow-seat': showMonthSelect || showNoonSelect
                            },
                            on: {
                                'click': function click() {
                                    return _this2.onViewChange('noon');
                                }
                            }
                        },
                        [currentNoon]
                    ) : null]
                ), h(
                    _button2['default'],
                    {
                        'class': 'sn-datetime-picker-view-btn',
                        attrs: { type: 'link'
                        },
                        on: {
                            'click': this.submit
                        }
                    },
                    ['\u786E\u5B9A']
                )]
            );
        },


        /* 渲染日历 */
        renderCalendar: function renderCalendar() {
            var h = this.$createElement;
            var mode = this.mode,
                showCalender = this.showCalender,
                currentValue = this.currentValue,
                disabledDate = this.disabledDate,
                vertical = this.vertical,
                showLastMonth = this.showLastMonth,
                showNextMonth = this.showNextMonth,
                workDate = this.workDate,
                infoDate = this.infoDate,
                touchable = this.touchable,
                mouseable = this.mouseable;


            if (mode === 'date' || mode === 'datetime') {
                return h(_calendar2['default'], {
                    ref: 'calendar',
                    directives: [{
                        name: 'show',
                        value: showCalender
                    }],
                    attrs: {
                        value: currentValue,
                        disabledDate: disabledDate,
                        vertical: vertical,
                        showLastMonth: showLastMonth,
                        showNextMonth: showNextMonth,
                        workDate: workDate,
                        infoDate: infoDate,
                        touchable: touchable,
                        mouseable: mouseable
                    },
                    on: {
                        'calendar-change': this.onCalendarChange,
                        'select': this.select
                    }
                });
            }
        },


        /* 渲染年月选择器 */
        renderYearMonthPicker: function renderYearMonthPicker() {
            var h = this.$createElement;
            var mode = this.mode,
                showMonthSelect = this.showMonthSelect,
                yearAndMonthList = this.yearAndMonthList;


            if (mode === 'date' || mode === 'datetime' || mode === 'year-month' || mode === 'year' || mode === 'month') {
                return h(
                    'div',
                    {
                        'class': {
                            'date-select': true,
                            'single-col': mode === 'year-month' || mode === 'year' || mode === 'month'
                        },
                        directives: [{
                            name: 'show',
                            value: showMonthSelect
                        }]
                    },
                    [h(_pickerView2['default'], {
                        ref: 'yearMonthPicker',
                        attrs: { visibleItemCount: 7,
                            columns: yearAndMonthList
                        },
                        on: {
                            'change': this.onYearAndMonthChange
                        }
                    })]
                );
            }
        },


        /* 渲染时分选择器 */
        renderTimePicker: function renderTimePicker() {
            var h = this.$createElement;
            var mode = this.mode,
                showTimeSelect = this.showTimeSelect,
                times = this.times;


            if (mode === 'datetime' || mode === 'time') {
                return h(
                    'div',
                    { 'class': 'time-select', directives: [{
                            name: 'show',
                            value: showTimeSelect
                        }]
                    },
                    [h(_pickerView2['default'], {
                        ref: 'timePicker',
                        attrs: { visibleItemCount: 7,
                            columns: times
                        },
                        on: {
                            'change': this.onTimeChange
                        }
                    })]
                );
            }
        },


        /* 渲染上下午选择器 */
        renderNoonPicker: function renderNoonPicker() {
            var h = this.$createElement;
            var mode = this.mode,
                use12Hours = this.use12Hours,
                showNoonSelect = this.showNoonSelect,
                noons = this.noons;


            if (mode === 'date' && use12Hours) {
                return h(
                    'div',
                    { 'class': 'noon-select', directives: [{
                            name: 'show',
                            value: showNoonSelect
                        }]
                    },
                    [h(_pickerView2['default'], {
                        ref: 'noonPicker',
                        attrs: { visibleItemCount: 7,
                            columns: noons
                        },
                        on: {
                            'change': this.onNoonChange
                        }
                    })]
                );
            }
        }
    },

    render: function render() {
        var h = arguments[0];
        var styleHeight = this.styleHeight;


        return h(
            'div',
            { 'class': 'sn-datetime-picker-view' },
            [this.renderHeader(), h(
                'div',
                {
                    'class': 'sn-datetime-picker-view-content',
                    style: { height: styleHeight }
                },
                [this.renderCalendar(), this.renderYearMonthPicker(), this.renderTimePicker(), this.renderNoonPicker()]
            )]
        );
    }
};

DateTimePickerView.install = function (Vue) {
    Vue.component(DateTimePickerView.name, DateTimePickerView);
};

exports['default'] = DateTimePickerView;
module.exports = exports.default;