'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _rangePickerView = require('../range-picker-view');

var _rangePickerView2 = _interopRequireDefault(_rangePickerView);

var _popup = require('../popup');

var _popup2 = _interopRequireDefault(_popup);

var _transferDom = require('../directives/transfer-dom');

var _transferDom2 = _interopRequireDefault(_transferDom);

var _dateUtil = require('../utils/dateUtil');

require('./index.less');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var RangePicker = {
    name: 'SnRangePicker',

    data: function data() {
        return {
            show: false,
            currentValue: '',
            key: 0
        };
    },

    directives: { TransferDom: _transferDom2['default'] },
    props: {
        value: [String, Number, Array, Date],
        readonly: Boolean,
        mode: {
            type: String,
            'default': 'date'
        },
        separator: {
            type: String,
            'default': '-'
        },
        disabledDate: Function,
        placeholder: {
            type: String,
            'default': '请选择'
        }
    },
    computed: {
        formatValue: function formatValue() {
            var currentValue = this.currentValue,
                separator = this.separator;


            if (!currentValue) return '';

            if (Array.isArray(currentValue)) {
                var now = new Date();

                if (currentValue.length === 0) {
                    return (0, _dateUtil.dateFormat)(now, 'YYYY/MM/DD') + ' ' + separator + ' ' + (0, _dateUtil.dateFormat)(now, 'YYYY/MM/DD');
                } else if (currentValue.length === 1) {
                    return (0, _dateUtil.dateFormat)(currentValue[0], 'YYYY/MM/DD') + ' ' + separator + ' ' + (0, _dateUtil.dateFormat)(now, 'YYYY/MM/DD');
                } else if (currentValue.length === 2) {
                    return (0, _dateUtil.dateFormat)(currentValue[0], 'YYYY/MM/DD') + ' ' + separator + ' ' + (0, _dateUtil.dateFormat)(currentValue[1], 'YYYY/MM/DD');
                } else {
                    console.error('[sinosun-ui] RangePicker 参数最多为2个的数组');
                }
            } else {
                console.error('[sinosun-ui] RangePicker 参数可选类型为空或者数组');
            }
        }
    },
    created: function created() {
        this.currentValue = this.value;
    },
    mounted: function mounted() {},

    methods: {
        onInput: function onInput(val) {
            this.show = val;
        },
        onOpen: function onOpen() {
            this.key++;
            this.$emit('open');
        },
        onOpened: function onOpened() {
            this.$emit('opened');
        },
        onClose: function onClose() {
            this.$emit('close');
        },
        onClosed: function onClosed() {
            this.$emit('closed');
        },
        onClickOverlay: function onClickOverlay() {
            this.show = false;
            this.$emit('click-overlay');
        },
        onClick: function onClick() {
            if (!this.readonly) {
                this.show = true;
            }
        },
        onChange: function onChange(val) {
            if (!this.$listeners['change']) {
                this.$emit('input', val);
            }

            this.show = false;
            this.$emit('change', val);
        }
    },
    watch: {
        value: function value(newVal, oldVal) {
            this.currentValue = newVal;
            this.key++;
        }
    },
    render: function render() {
        var h = arguments[0];
        var currentValue = this.currentValue,
            formatValue = this.formatValue,
            key = this.key,
            show = this.show,
            disabledDate = this.disabledDate,
            placeholder = this.placeholder;


        return h(
            'div',
            { 'class': 'sn-range-picker', on: {
                    'click': this.onClick
                }
            },
            [formatValue ? h(
                'span',
                { 'class': 'sn-range-picker-value' },
                [formatValue]
            ) : h(
                'span',
                { 'class': 'sn-range-picker-placeholder' },
                [placeholder]
            ), h(
                'div',
                {
                    directives: [{
                        name: 'transfer-dom',
                        value: true
                    }]
                },
                [h(
                    _popup2['default'],
                    {
                        attrs: {
                            value: show
                        },
                        on: {
                            'input': this.onInput,
                            'open': this.onOpen,
                            'opened': this.onOpened,
                            'close': this.onClose,
                            'closed': this.onClosed,
                            'click-overlay': this.onClickOverlay
                        },

                        style: 'background: transparent;'
                    },
                    [h(
                        'div',
                        { 'class': 'sn-range-picker-content', key: key },
                        [h(_rangePickerView2['default'], {
                            attrs: {
                                value: currentValue,
                                disabledDate: disabledDate
                            },
                            on: {
                                'change': this.onChange
                            }
                        })]
                    )]
                )]
            )]
        );
    }
};

RangePicker.install = function (Vue) {
    Vue.component(RangePicker.name, RangePicker);
};

exports['default'] = RangePicker;
module.exports = exports.default;